@@ -32,6 +32,7 @@
 #include "user_interface.h"
 #include "ets_alt_task.h"
 #include "py/runtime.h"
+#include "py/stream.h"
 #include "extmod/misc.h"
 #include "lib/utils/pyexec.h"
 
@@ -56,6 +57,14 @@ void mp_hal_delay_us(uint32_t us) {
     }
 }
 
+uintptr_t mp_hal_stdio_poll(uintptr_t poll_flags) {
+    uintptr_t ret = 0;
+    if ((poll_flags & MP_STREAM_POLL_RD) && stdin_ringbuf.iget != stdin_ringbuf.iput) {
+        ret |= MP_STREAM_POLL_RD;
+    }
+    return ret;
+}
+
 int mp_hal_stdin_rx_chr(void) {
     for (;;) {
         int c = ringbuf_get(&stdin_ringbuf);
@@ -140,8 +149,7 @@ void ets_event_poll(void) {
 
 void __assert_func(const char *file, int line, const char *func, const char *expr) {
     printf("assert:%s:%d:%s: %s\n", file, line, func, expr);
-    nlr_raise(mp_obj_new_exception_msg(&mp_type_AssertionError,
-        "C-level assert"));
+    mp_raise_msg(&mp_type_AssertionError, "C-level assert");
 }
 
 void mp_hal_signal_input(void) {
