@@ -62,7 +62,7 @@ STATIC void microbit_image_print(const mp_print_t *print, mp_obj_t self_in, mp_p
 
 uint8_t monochromeGetPixelValue(monochrome_5by5_t * p_mono, mp_int_t x, mp_int_t y) {
     unsigned int index = y*5+x;
-    if (index == 24) 
+    if (index == 24)
         return p_mono->pixel44;
     return (p_mono->bits24[index>>3] >> (index&7))&1;
 }
@@ -227,8 +227,7 @@ STATIC mp_obj_t microbit_image_make_new(const mp_obj_type_t *type_in, mp_uint_t
                     return image_from_parsed_str(str, len);
                 }
             } else {
-                nlr_raise(mp_obj_new_exception_msg(&mp_type_TypeError,
-                    "Image(s) takes a string."));
+                mp_raise_msg(&mp_type_TypeError, "Image(s) takes a string.");
             }
         }
 
@@ -259,8 +258,7 @@ STATIC mp_obj_t microbit_image_make_new(const mp_obj_type_t *type_in, mp_uint_t
         }
 
         default: {
-            nlr_raise(mp_obj_new_exception_msg(&mp_type_TypeError,
-                "Image() takes 0 to 3 arguments"));
+            mp_raise_msg(&mp_type_TypeError, "Image() takes 0 to 3 arguments");
         }
     }
 }
@@ -365,7 +363,7 @@ MP_DEFINE_CONST_FUN_OBJ_3(microbit_image_get_pixel_obj, microbit_image_get_pixel
 /* Raise an exception if not mutable */
 static void check_mutability(microbit_image_obj_t *self) {
     if (self->base.five) {
-        nlr_raise(mp_obj_new_exception_msg(&mp_type_TypeError, "image cannot be modified (try copying first)"));
+        mp_raise_msg(&mp_type_TypeError, "image cannot be modified (try copying first)");
     }
 }
 
@@ -380,7 +378,7 @@ mp_obj_t microbit_image_set_pixel(mp_uint_t n_args, const mp_obj_t *args) {
         mp_raise_ValueError("index cannot be negative");
     }
     mp_int_t bright = mp_obj_get_int(args[3]);
-    if (bright < 0 || bright > MAX_BRIGHTNESS) 
+    if (bright < 0 || bright > MAX_BRIGHTNESS)
         mp_raise_ValueError("brightness out of bounds.");
     if (x < imageWidth(self) && y < imageHeight(self)) {
         greyscaleSetPixelValue(&(self->greyscale), x, y, bright);
@@ -408,11 +406,10 @@ mp_obj_t microbit_image_blit(mp_uint_t n_args, const mp_obj_t *args) {
 
     mp_obj_t src = args[1];
     if (mp_obj_get_type(src) != &microbit_image_type) {
-        nlr_raise(mp_obj_new_exception_msg(&mp_type_TypeError, "expecting an image"));
+        mp_raise_msg(&mp_type_TypeError, "expecting an image");
     }
     if (n_args == 7) {
-        nlr_raise(mp_obj_new_exception_msg(&mp_type_TypeError,
-            "must specify both offsets"));
+        mp_raise_msg(&mp_type_TypeError, "must specify both offsets");
     }
     mp_int_t x = mp_obj_get_int(args[2]);
     mp_int_t y = mp_obj_get_int(args[3]);
@@ -610,7 +607,7 @@ microbit_image_obj_t *microbit_image_dim(microbit_image_obj_t *lhs, mp_float_t f
 #else
 microbit_image_obj_t *microbit_image_dim(microbit_image_obj_t *lhs, mp_int_t fval) {
 #endif
-    if (fval < 0) 
+    if (fval < 0)
         mp_raise_ValueError("Brightness multiplier must not be negative.");
     greyscale_t *result = greyscale_new(imageWidth(lhs), imageHeight(lhs));
     for (int x = 0; x < imageWidth(lhs); ++x) {
@@ -639,7 +636,7 @@ microbit_image_obj_t *microbit_image_sum(microbit_image_obj_t *lhs, microbit_ima
             int val;
             int lval = imageGetPixelValue(lhs, x,y);
             int rval = imageGetPixelValue(rhs, x,y);
-            if (add) 
+            if (add)
                 val = min(lval + rval, MAX_BRIGHTNESS);
             else
                 val = max(0, lval - rval);
@@ -647,7 +644,7 @@ microbit_image_obj_t *microbit_image_sum(microbit_image_obj_t *lhs, microbit_ima
         }
     }
     return (microbit_image_obj_t *)result;
-}                      
+}
 
 STATIC mp_obj_t image_binary_op(mp_binary_op_t op, mp_obj_t lhs_in, mp_obj_t rhs_in) {
     if (mp_obj_get_type(lhs_in) != &microbit_image_type) {
@@ -697,9 +694,9 @@ const mp_obj_type_t microbit_image_type = {
     .buffer_p = {NULL},
     .locals_dict = (mp_obj_dict_t*)&microbit_image_locals_dict,
 };
- 
+
 typedef struct _scrolling_string_t {
-    mp_obj_base_t base; 
+    mp_obj_base_t base;
     char const *str;
     mp_uint_t len;
     mp_obj_t ref;
@@ -708,7 +705,7 @@ typedef struct _scrolling_string_t {
 } scrolling_string_t;
 
 typedef struct _scrolling_string_iterator_t {
-    mp_obj_base_t base; 
+    mp_obj_base_t base;
     mp_obj_t ref;
     greyscale_t *img;
     char const *next_char;
@@ -962,7 +959,7 @@ const mp_obj_type_t microbit_facade_iterator_type = {
 };
 
 mp_obj_t microbit_facade_iterator(mp_obj_t iterable_in, mp_obj_iter_buf_t *iter_buf) {
-	(void)iter_buf;
+    (void)iter_buf;
     facade_iterator_t *result = m_new_obj(facade_iterator_t);
     string_image_facade_t *iterable = (string_image_facade_t *)iterable_in;
     result->base.type = &microbit_facade_iterator_type;
