@@ -10,7 +10,7 @@ void NETDUINO_PLUS_2_board_early_init(void) {
     GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
     GPIO_InitStructure.Pull = GPIO_PULLUP;
 
-#if MICROPY_HW_HAS_SDCARD
+#if MICROPY_HW_ENABLE_SDCARD
     // Turn on the power enable for the sdcard (PB1)
     GPIO_InitStructure.Pin = GPIO_PIN_1;
     HAL_GPIO_Init(GPIOB, &GPIO_InitStructure);
