@@ -1,7 +1,5 @@
-// This board is only confirmed to operate using DFU mode and openocd.
-// DFU mode can be accessed by setting BOOT0 (see schematics)
-// To use openocd run "OPENOCD_CONFIG=boards/openocd_stm32f7.cfg" in
-// the make command.
+// Note: if the board shows odd behaviour check the option bits and make sure nDBANK is
+// set to make the 2MByte space continuous instead of divided into two 1MByte segments.
 
 #define MICROPY_HW_BOARD_NAME       "NUCLEO-F767ZI"
 #define MICROPY_HW_MCU_NAME         "STM32F767"
@@ -12,6 +10,7 @@
 #define MICROPY_HW_ENABLE_RTC       (1)
 #define MICROPY_HW_ENABLE_DAC       (1)
 #define MICROPY_HW_ENABLE_USB       (1)
+#define MICROPY_HW_ENABLE_SDCARD    (1)
 
 #define MICROPY_BOARD_EARLY_INIT    NUCLEO_F767ZI_board_early_init
 void NUCLEO_F767ZI_board_early_init(void);
@@ -30,16 +29,25 @@ void NUCLEO_F767ZI_board_early_init(void);
 #define MICROPY_HW_UART2_CTS        (pin_D3)
 #define MICROPY_HW_UART3_TX         (pin_D8)
 #define MICROPY_HW_UART3_RX         (pin_D9)
-#define MICROPY_HW_UART6_TX         (pin_G14)
-#define MICROPY_HW_UART6_RX         (pin_G9)
+#define MICROPY_HW_UART6_TX         (pin_C6)
+#define MICROPY_HW_UART6_RX         (pin_C7)
+#define MICROPY_HW_UART5_TX         (pin_B6)
+#define MICROPY_HW_UART5_RX         (pin_B12)
+#define MICROPY_HW_UART7_TX         (pin_F7)
+#define MICROPY_HW_UART7_RX         (pin_F6)
+#define MICROPY_HW_UART8_TX         (pin_E1)
+#define MICROPY_HW_UART8_RX         (pin_E0)
+
 #define MICROPY_HW_UART_REPL        PYB_UART_3
 #define MICROPY_HW_UART_REPL_BAUD   115200
 
 // I2C busses
 #define MICROPY_HW_I2C1_SCL         (pin_B8)
 #define MICROPY_HW_I2C1_SDA         (pin_B9)
-#define MICROPY_HW_I2C3_SCL         (pin_H7)
-#define MICROPY_HW_I2C3_SDA         (pin_H8)
+#define MICROPY_HW_I2C2_SCL         (pin_F1)
+#define MICROPY_HW_I2C2_SDA         (pin_F0)
+#define MICROPY_HW_I2C4_SCL         (pin_F14)
+#define MICROPY_HW_I2C4_SDA         (pin_F15)
 
 // SPI
 #define MICROPY_HW_SPI3_NSS         (pin_A4)
@@ -48,10 +56,8 @@ void NUCLEO_F767ZI_board_early_init(void);
 #define MICROPY_HW_SPI3_MOSI        (pin_B5)
 
 // CAN busses
-#define MICROPY_HW_CAN1_TX          (pin_B9)
-#define MICROPY_HW_CAN1_RX          (pin_B8)
-#define MICROPY_HW_CAN2_TX          (pin_B13)
-#define MICROPY_HW_CAN2_RX          (pin_B12)
+#define MICROPY_HW_CAN1_TX          (pin_D1)
+#define MICROPY_HW_CAN1_RX          (pin_D0)
 
 // USRSW is pulled low. Pressing the button makes the input go high.
 #define MICROPY_HW_USRSW_PIN        (pin_C13)
@@ -71,6 +77,11 @@ void NUCLEO_F767ZI_board_early_init(void);
 #define MICROPY_HW_USB_VBUS_DETECT_PIN (pin_A9)
 #define MICROPY_HW_USB_OTG_ID_PIN      (pin_A10)
 
+// SD card detect switch (actual pin may need to be changed for a particular use)
+#define MICROPY_HW_SDCARD_DETECT_PIN        (pin_G2)
+#define MICROPY_HW_SDCARD_DETECT_PULL       (GPIO_PULLUP)
+#define MICROPY_HW_SDCARD_DETECT_PRESENT    (GPIO_PIN_RESET)
+
 // Ethernet via RMII
 #define MICROPY_HW_ETH_MDC          (pin_C1)
 #define MICROPY_HW_ETH_MDIO         (pin_A2)
