@@ -31,8 +31,21 @@ void NUCLEO_H743ZI_board_early_init(void);
 #define MICROPY_HW_FLASH_LATENCY    FLASH_LATENCY_4
 
 // UART config
+#define MICROPY_HW_UART2_TX         (pin_D5)
+#define MICROPY_HW_UART2_RX         (pin_D6)
+#define MICROPY_HW_UART2_RTS        (pin_D4)
+#define MICROPY_HW_UART2_CTS        (pin_D3)
 #define MICROPY_HW_UART3_TX         (pin_D8)
 #define MICROPY_HW_UART3_RX         (pin_D9)
+#define MICROPY_HW_UART5_TX         (pin_B6)
+#define MICROPY_HW_UART5_RX         (pin_B12)
+#define MICROPY_HW_UART6_TX         (pin_C6)
+#define MICROPY_HW_UART6_RX         (pin_C7)
+#define MICROPY_HW_UART7_TX         (pin_F7)
+#define MICROPY_HW_UART7_RX         (pin_F6)
+#define MICROPY_HW_UART8_TX         (pin_E1)
+#define MICROPY_HW_UART8_RX         (pin_E0)
+
 #define MICROPY_HW_UART_REPL        PYB_UART_3
 #define MICROPY_HW_UART_REPL_BAUD   115200
 
@@ -41,6 +54,8 @@ void NUCLEO_H743ZI_board_early_init(void);
 #define MICROPY_HW_I2C1_SDA         (pin_B9)
 #define MICROPY_HW_I2C2_SCL         (pin_F1)
 #define MICROPY_HW_I2C2_SDA         (pin_F0)
+#define MICROPY_HW_I2C4_SCL         (pin_F14)
+#define MICROPY_HW_I2C4_SDA         (pin_F15)
 
 // SPI
 #define MICROPY_HW_SPI3_NSS         (pin_A4)
@@ -66,7 +81,23 @@ void NUCLEO_H743ZI_board_early_init(void);
 #define MICROPY_HW_USB_VBUS_DETECT_PIN (pin_A9)
 #define MICROPY_HW_USB_OTG_ID_PIN      (pin_A10)
 
+// FDCAN bus
+#define MICROPY_HW_CAN1_NAME  "FDCAN1"
+#define MICROPY_HW_CAN1_TX    (pin_D1)
+#define MICROPY_HW_CAN1_RX    (pin_D0)
+
 // SD card detect switch
 #define MICROPY_HW_SDCARD_DETECT_PIN        (pin_G2)
 #define MICROPY_HW_SDCARD_DETECT_PULL       (GPIO_PULLUP)
 #define MICROPY_HW_SDCARD_DETECT_PRESENT    (GPIO_PIN_RESET)
+
+// Ethernet via RMII (MDC define disabled for now until eth.c supports H7)
+//#define MICROPY_HW_ETH_MDC          (pin_C1)
+#define MICROPY_HW_ETH_MDIO         (pin_A2)
+#define MICROPY_HW_ETH_RMII_REF_CLK (pin_A1)
+#define MICROPY_HW_ETH_RMII_CRS_DV  (pin_A7)
+#define MICROPY_HW_ETH_RMII_RXD0    (pin_C4)
+#define MICROPY_HW_ETH_RMII_RXD1    (pin_C5)
+#define MICROPY_HW_ETH_RMII_TX_EN   (pin_G11)
+#define MICROPY_HW_ETH_RMII_TXD0    (pin_G13)
+#define MICROPY_HW_ETH_RMII_TXD1    (pin_B13)
