@@ -86,6 +86,13 @@ STATIC byte flash_cache_mem[0x4000] __attribute__((aligned(4))); // 16k
 #define FLASH_MEM_SEG2_START_ADDR (0x08140000) // sector 18
 #define FLASH_MEM_SEG2_NUM_BLOCKS (128) // sector 18: 64k(of 128k)
 
+#elif defined(STM32F722xx) || defined(STM32F723xx) || defined(STM32F732xx) || defined(STM32F733xx)
+
+#define CACHE_MEM_START_ADDR (0x20000000) // DTCM data RAM, 64k
+#define FLASH_SECTOR_SIZE_MAX (0x10000) // 64k max
+#define FLASH_MEM_SEG1_START_ADDR (0x08004000) // sector 1
+#define FLASH_MEM_SEG1_NUM_BLOCKS (224) // sectors 1,2,3,4: 16k+16k+16k+64k=112k
+
 #elif defined(STM32F746xx) || defined(STM32F765xx) || defined(STM32F767xx) || defined(STM32F769xx)
 
 // The STM32F746 doesn't really have CCRAM, so we use the 64K DTCM for this.
@@ -103,9 +110,12 @@ STATIC byte flash_cache_mem[0x4000] __attribute__((aligned(4))); // 16k
 #define FLASH_MEM_SEG1_START_ADDR (0x08020000) // sector 1
 #define FLASH_MEM_SEG1_NUM_BLOCKS (256) // Sector 1: 128k / 512b = 256 blocks
 
-#elif defined(STM32L432xx) || defined(STM32L475xx) || defined(STM32L476xx) || defined(STM32L496xx)
+#elif defined(STM32L432xx) || \
+      defined(STM32L451xx) || defined(STM32L452xx) || defined(STM32L462xx) || \
+      defined(STM32L475xx) || defined(STM32L476xx) || defined(STM32L496xx) || \
+      defined(STM32WB)
 
-// The STM32L475/6 doesn't have CCRAM, so we use the 32K SRAM2 for this, although
+// The STM32L4xx doesn't have CCRAM, so we use SRAM2 for this, although
 // actual location and size is defined by the linker script.
 extern uint8_t _flash_fs_start;
 extern uint8_t _flash_fs_end;
@@ -282,4 +292,44 @@ bool flash_bdev_writeblock(const uint8_t *src, uint32_t block) {
     return true;
 }
 
+int flash_bdev_readblocks_ext(uint8_t *dest, uint32_t block, uint32_t offset, uint32_t len) {
+    // Get data from flash memory, possibly via cache
+    while (len) {
+        uint32_t l = MIN(len, FLASH_BLOCK_SIZE - offset);
+        uint32_t flash_addr = convert_block_to_flash_addr(block);
+        if (flash_addr == -1) {
+            // bad block number
+            return -1;
+        }
+        uint8_t *src = flash_cache_get_addr_for_read(flash_addr + offset);
+        memcpy(dest, src, l);
+        dest += l;
+        block += 1;
+        offset = 0;
+        len -= l;
+    }
+    return 0;
+}
+
+int flash_bdev_writeblocks_ext(const uint8_t *src, uint32_t block, uint32_t offset, uint32_t len) {
+    // Copy to cache
+    while (len) {
+        uint32_t l = MIN(len, FLASH_BLOCK_SIZE - offset);
+        uint32_t flash_addr = convert_block_to_flash_addr(block);
+        if (flash_addr == -1) {
+            // bad block number
+            return -1;
+        }
+        uint32_t basepri = raise_irq_pri(IRQ_PRI_FLASH); // prevent cache flushing and USB access
+        uint8_t *dest = flash_cache_get_addr_for_write(flash_addr + offset);
+        memcpy(dest, src, l);
+        restore_irq_pri(basepri);
+        src += l;
+        block += 1;
+        offset = 0;
+        len -= l;
+    }
+    return 0;
+}
+
 #endif // MICROPY_HW_ENABLE_INTERNAL_FLASH_STORAGE
