@@ -28,6 +28,9 @@
 
 #include "py/obj.h"
 
+extern const mp_obj_type_t machine_adc_type;
+extern const mp_obj_type_t machine_timer_type;
+
 void machine_init(void);
 void machine_deinit(void);
 
