@@ -39,17 +39,14 @@
 
 struct netif;
 
-typedef struct _mod_network_nic_type_t {
-    mp_obj_base_t base;
-    void (*poll_callback)(void *data, struct netif *netif);
-} mod_network_nic_type_t;
-
 extern const mp_obj_type_t network_lan_type;
 extern const mp_obj_type_t mod_network_nic_type_wiznet5k;
 
 void mod_network_lwip_poll_wrapper(uint32_t ticks_ms);
 mp_obj_t mod_network_nic_ifconfig(struct netif *netif, size_t n_args, const mp_obj_t *args);
 
+void wiznet5k_poll(void);
+
 #else
 
 struct _mod_network_socket_obj_t;
