@@ -26,7 +26,7 @@
 
 #include <stdio.h>
 
-#include "py/mpstate.h"
+#include "py/runtime.h"
 #include "py/gc.h"
 #include "py/mpthread.h"
 #include "gccollect.h"
@@ -75,7 +75,7 @@ void mp_thread_create(void *(*entry)(void*), void *arg, size_t *stack_size) {
     uint32_t id = pyb_thread_new(th, stack, stack_len, entry, arg);
     if (id == 0) {
         mp_thread_mutex_unlock(&thread_mutex);
-        nlr_raise(mp_obj_new_exception_msg(&mp_type_OSError, "can't create thread"));
+        mp_raise_msg(&mp_type_OSError, "can't create thread");
     }
 
     mp_thread_mutex_unlock(&thread_mutex);
