@@ -31,7 +31,7 @@
 #include "usbd_desc.h"
 #include "usbd_cdc_msc_hid.h"
 #include "usbd_cdc_interface.h"
-#include "usbd_msc_storage.h"
+#include "usbd_msc_interface.h"
 #include "usbd_hid_interface.h"
 
 #include "py/objstr.h"
@@ -40,6 +40,8 @@
 #include "py/mperrno.h"
 #include "py/mphal.h"
 #include "bufhelper.h"
+#include "storage.h"
+#include "sdcard.h"
 #include "usb.h"
 
 #if MICROPY_HW_ENABLE_USB
@@ -55,6 +57,21 @@
 #endif
 #endif
 
+// Maximum number of endpoints (excluding EP0)
+#if defined(STM32L0) || defined(STM32WB)
+#define MAX_ENDPOINT(dev_id) (7)
+#elif defined(STM32L4)
+#define MAX_ENDPOINT(dev_id) (5)
+#elif defined(STM32F4)
+#define MAX_ENDPOINT(dev_id) ((dev_id) == USB_PHY_FS_ID ? 3 : 5)
+#elif defined(STM32F7)
+#define MAX_ENDPOINT(dev_id) ((dev_id) == USB_PHY_FS_ID ? 5 : 8)
+#elif defined(STM32H7)
+#define MAX_ENDPOINT(dev_id) (8)
+#endif
+
+STATIC void pyb_usb_vcp_init0(void);
+
 // this will be persistent across a soft-reset
 mp_uint_t pyb_usb_flags = 0;
 
@@ -62,16 +79,84 @@ typedef struct _usb_device_t {
     uint32_t enabled;
     USBD_HandleTypeDef hUSBDDevice;
     usbd_cdc_msc_hid_state_t usbd_cdc_msc_hid_state;
-    usbd_cdc_itf_t usbd_cdc_itf;
-    #if MICROPY_HW_USB_ENABLE_CDC2
-    usbd_cdc_itf_t usbd_cdc2_itf;
-    #endif
+    usbd_cdc_itf_t usbd_cdc_itf[MICROPY_HW_USB_CDC_NUM];
+    #if MICROPY_HW_USB_HID
     usbd_hid_itf_t usbd_hid_itf;
+    #endif
 } usb_device_t;
 
 usb_device_t usb_device = {0};
 pyb_usb_storage_medium_t pyb_usb_storage_medium = PYB_USB_STORAGE_MEDIUM_NONE;
 
+#if !MICROPY_HW_USB_IS_MULTI_OTG
+
+// Units of FIFO size arrays below are 4x 16-bit words = 8 bytes
+// There are 512x 16-bit words it total to use here (when using PCD_SNG_BUF)
+
+// EP0(out), EP0(in), MSC/HID(out), MSC/HID(in), unused, CDC_CMD(in), CDC_DATA(out), CDC_DATA(in)
+STATIC const uint8_t usbd_fifo_size_cdc1[] = {16, 16, 16, 16, 0, 16, 16, 16};
+
+#else
+
+// Units of FIFO size arrays below are 4x 32-bit words = 16 bytes
+// FS: there are 320x 32-bit words in total to use here
+// HS: there are 1024x 32-bit words in total to use here
+
+// RX; EP0(in), MSC/HID, CDC_CMD, CDC_DATA
+STATIC const uint8_t usbd_fifo_size_cdc1[] = {
+    32, 8, 16, 8, 16, 0, 0, // FS: RX, EP0(in), 5x IN endpoints
+    #if MICROPY_HW_USB_HS
+    116, 8, 64, 4, 64, 0, 0, 0, 0, 0, // HS: RX, EP0(in), 8x IN endpoints
+    #endif
+};
+
+// RX; EP0(in), MSC/HID, CDC_CMD, CDC_DATA, HID
+STATIC const uint8_t usbd_fifo_size_cdc1_msc_hid[] = {
+    32, 8, 16, 4, 12, 8, 0,
+    #if MICROPY_HW_USB_HS
+    116, 8, 64, 4, 56, 8, 0, 0, 0, 0,
+    #endif
+};
+
+#if MICROPY_HW_USB_CDC_NUM >= 2
+// RX; EP0(in), MSC/HID, CDC_CMD, CDC_DATA, CDC2_CMD, CDC2_DATA
+STATIC const uint8_t usbd_fifo_size_cdc2[] = {
+    32, 8, 16, 4, 8, 4, 8,
+    #if MICROPY_HW_USB_HS
+    116, 8, 64, 2, 32, 2, 32, 0, 0, 0,
+    #endif
+};
+
+// RX; EP0(in), MSC/HID, CDC_CMD, CDC_DATA, CDC2_CMD/HID, CDC2_DATA, HID
+STATIC const uint8_t usbd_fifo_size_cdc2_msc_hid[] = {
+    0, 0, 0, 0, 0, 0, 0, // FS: can't support 2xVCP+MSC+HID
+    #if MICROPY_HW_USB_HS
+    102, 8, 64, 2, 32, 8, 32, 8, 0, 0,
+    #endif
+};
+#endif
+
+#if MICROPY_HW_USB_CDC_NUM >= 3
+// RX; EP0(in), MSC/HID, CDC_CMD, CDC_DATA, CDC2_CMD, CDC2_DATA, CDC3_CMD, CDC3_DATA
+STATIC const uint8_t usbd_fifo_size_cdc3[] = {
+    0, 0, 0, 0, 0, 0, 0, // FS: can't support 3x VCP mode
+    #if MICROPY_HW_USB_HS
+    82, 8, 64, 2, 32, 2, 32, 2, 32, 0,
+    #endif
+};
+
+// RX; EP0(in), MSC/HID, CDC_CMD, CDC_DATA, CDC2_CMD/HID, CDC2_DATA, CDC3_CMD/HID, CDC3_DATA, HID
+STATIC const uint8_t usbd_fifo_size_cdc3_msc_hid[] = {
+    0, 0, 0, 0, 0, 0, 0, // FS: can't support 3x VCP mode
+    #if MICROPY_HW_USB_HS
+    82, 8, 64, 2, 25, 8, 25, 8, 25, 8,
+    #endif
+};
+#endif
+
+#endif
+
+#if MICROPY_HW_USB_HID
 // predefined hid mouse data
 STATIC const mp_obj_str_t pyb_usb_hid_mouse_desc_obj = {
     {&mp_type_bytes},
@@ -109,55 +194,117 @@ const mp_rom_obj_tuple_t pyb_usb_hid_keyboard_obj = {
         MP_ROM_PTR(&pyb_usb_hid_keyboard_desc_obj),
     },
 };
+#endif
 
 void pyb_usb_init0(void) {
-    usb_device.usbd_cdc_itf.attached_to_repl = false;
-    #if MICROPY_HW_USB_ENABLE_CDC2
-    usb_device.usbd_cdc2_itf.attached_to_repl = false;
-    #endif
+    for (int i = 0; i < MICROPY_HW_USB_CDC_NUM; ++i) {
+        usb_device.usbd_cdc_itf[i].attached_to_repl = false;
+    }
     MP_STATE_PORT(pyb_hid_report_desc) = MP_OBJ_NULL;
+
+    pyb_usb_vcp_init0();
 }
 
-bool pyb_usb_dev_init(uint16_t vid, uint16_t pid, usb_device_mode_t mode, USBD_HID_ModeInfoTypeDef *hid_info) {
-    bool high_speed = (mode & USBD_MODE_HIGH_SPEED) != 0;
-    mode &= 0x7f;
+int pyb_usb_dev_detect(void) {
+    if (usb_device.enabled) {
+        return usb_device.hUSBDDevice.id;
+    }
+
+    #if MICROPY_HW_USB_FS && MICROPY_HW_USB_HS
+    // Try to auto-detect which USB is connected by reading DP/DM pins
+    for (int i = 0; i < 2; ++i) {
+        mp_hal_pin_obj_t dp = i == 0 ? pyb_pin_USB_DP : pyb_pin_USB_HS_DP;
+        mp_hal_pin_obj_t dm = i == 0 ? pyb_pin_USB_DM : pyb_pin_USB_HS_DM;
+        mp_hal_pin_config(dp, MP_HAL_PIN_MODE_INPUT, MP_HAL_PIN_PULL_UP, 0);
+        mp_hal_pin_config(dm, MP_HAL_PIN_MODE_INPUT, MP_HAL_PIN_PULL_UP, 0);
+        int state = mp_hal_pin_read(dp) == 0 && mp_hal_pin_read(dm) == 0;
+        mp_hal_pin_config(dp, MP_HAL_PIN_MODE_INPUT, MP_HAL_PIN_PULL_NONE, 0);
+        mp_hal_pin_config(dm, MP_HAL_PIN_MODE_INPUT, MP_HAL_PIN_PULL_NONE, 0);
+        if (state) {
+            // DP and DM pins are actively held low so assume USB is connected
+            return i == 0 ? USB_PHY_FS_ID : USB_PHY_HS_ID;
+        }
+    }
+    #endif
+
+    return MICROPY_HW_USB_MAIN_DEV;
+}
+
+bool pyb_usb_dev_init(int dev_id, uint16_t vid, uint16_t pid, uint8_t mode, size_t msc_n, const void *msc_unit, USBD_HID_ModeInfoTypeDef *hid_info) {
     usb_device_t *usb_dev = &usb_device;
     if (!usb_dev->enabled) {
         // only init USB once in the device's power-lifetime
 
         // set up the USBD state
         USBD_HandleTypeDef *usbd = &usb_dev->hUSBDDevice;
-        usbd->id = MICROPY_HW_USB_MAIN_DEV;
+        usbd->id = dev_id;
         usbd->dev_state  = USBD_STATE_DEFAULT;
         usbd->pDesc = (USBD_DescriptorsTypeDef*)&USBD_Descriptors;
         usbd->pClass = &USBD_CDC_MSC_HID;
         usb_dev->usbd_cdc_msc_hid_state.pdev = usbd;
-        usb_dev->usbd_cdc_msc_hid_state.cdc = &usb_dev->usbd_cdc_itf.base;
-        #if MICROPY_HW_USB_ENABLE_CDC2
-        usb_dev->usbd_cdc_msc_hid_state.cdc2 = &usb_dev->usbd_cdc2_itf.base;
-        #endif
+        for (int i = 0; i < MICROPY_HW_USB_CDC_NUM; ++i) {
+            usb_dev->usbd_cdc_msc_hid_state.cdc[i] = &usb_dev->usbd_cdc_itf[i].base;
+        }
+        #if MICROPY_HW_USB_HID
         usb_dev->usbd_cdc_msc_hid_state.hid = &usb_dev->usbd_hid_itf.base;
+        #endif
         usbd->pClassData = &usb_dev->usbd_cdc_msc_hid_state;
 
         // configure the VID, PID and the USBD mode (interfaces it will expose)
-        USBD_SetVIDPIDRelease(&usb_dev->usbd_cdc_msc_hid_state, vid, pid, 0x0200, mode == USBD_MODE_CDC);
-        if (USBD_SelectMode(&usb_dev->usbd_cdc_msc_hid_state, mode, hid_info) != 0) {
+        int cdc_only = (mode & USBD_MODE_IFACE_MASK) == USBD_MODE_CDC;
+        USBD_SetVIDPIDRelease(&usb_dev->usbd_cdc_msc_hid_state, vid, pid, 0x0200, cdc_only);
+        if (USBD_SelectMode(&usb_dev->usbd_cdc_msc_hid_state, mode, hid_info, MAX_ENDPOINT(dev_id)) != 0) {
             return false;
         }
 
-        switch (pyb_usb_storage_medium) {
-            #if MICROPY_HW_ENABLE_SDCARD
-            case PYB_USB_STORAGE_MEDIUM_SDCARD:
-                USBD_MSC_RegisterStorage(&usb_dev->usbd_cdc_msc_hid_state, (USBD_StorageTypeDef*)&USBD_SDCARD_STORAGE_fops);
-                break;
-            #endif
-            default:
-                USBD_MSC_RegisterStorage(&usb_dev->usbd_cdc_msc_hid_state, (USBD_StorageTypeDef*)&USBD_FLASH_STORAGE_fops);
-                break;
+        #if MICROPY_HW_USB_MSC
+        // Configure the MSC interface
+        const void *msc_unit_default[1];
+        if (msc_n == 0) {
+            msc_n = 1;
+            msc_unit = msc_unit_default;
+            switch (pyb_usb_storage_medium) {
+                #if MICROPY_HW_ENABLE_SDCARD
+                case PYB_USB_STORAGE_MEDIUM_SDCARD:
+                    msc_unit_default[0] = &pyb_sdcard_type;
+                    break;
+                #endif
+                default:
+                    msc_unit_default[0] = &pyb_flash_type;
+                    break;
+            }
         }
+        usbd_msc_init_lu(msc_n, msc_unit);
+        USBD_MSC_RegisterStorage(&usb_dev->usbd_cdc_msc_hid_state, (USBD_StorageTypeDef*)&usbd_msc_fops);
+        #endif
+
+        const uint8_t *fifo_size = usbd_fifo_size_cdc1;
+        #if MICROPY_HW_USB_IS_MULTI_OTG
+        if ((mode & USBD_MODE_MSC_HID) == USBD_MODE_MSC_HID) {
+            fifo_size = usbd_fifo_size_cdc1_msc_hid;
+        }
+        #endif
+        #if MICROPY_HW_USB_CDC_NUM >= 3
+        if (mode & USBD_MODE_IFACE_CDC(2)) {
+            if ((mode & USBD_MODE_MSC_HID) == USBD_MODE_MSC_HID) {
+                fifo_size = usbd_fifo_size_cdc3_msc_hid;
+            } else {
+                fifo_size = usbd_fifo_size_cdc3;
+            }
+        } else
+        #endif
+        #if MICROPY_HW_USB_CDC_NUM >= 2
+        if (mode & USBD_MODE_IFACE_CDC(1)) {
+            if ((mode & USBD_MODE_MSC_HID) == USBD_MODE_MSC_HID) {
+                fifo_size = usbd_fifo_size_cdc2_msc_hid;
+            } else {
+                fifo_size = usbd_fifo_size_cdc2;
+            }
+        }
+        #endif
 
         // start the USB device
-        USBD_LL_Init(usbd, high_speed);
+        USBD_LL_Init(usbd, (mode & USBD_MODE_HIGH_SPEED) != 0, fifo_size);
         USBD_LL_Start(usbd);
         usb_dev->enabled = true;
     }
@@ -179,22 +326,17 @@ bool usb_vcp_is_enabled(void) {
 }
 
 int usb_vcp_recv_byte(uint8_t *c) {
-    return usbd_cdc_rx(&usb_device.usbd_cdc_itf, c, 1, 0);
+    return usbd_cdc_rx(&usb_device.usbd_cdc_itf[0], c, 1, 0);
 }
 
 void usb_vcp_send_strn(const char *str, int len) {
     if (usb_device.enabled) {
-        usbd_cdc_tx_always(&usb_device.usbd_cdc_itf, (const uint8_t*)str, len);
+        usbd_cdc_tx_always(&usb_device.usbd_cdc_itf[0], (const uint8_t*)str, len);
     }
 }
 
 usbd_cdc_itf_t *usb_vcp_get(int idx) {
-    #if MICROPY_HW_USB_ENABLE_CDC2
-    if (idx == 1) {
-        return &usb_device.usbd_cdc2_itf;
-    }
-    #endif
-    return &usb_device.usbd_cdc_itf;
+    return &usb_device.usbd_cdc_itf[idx];
 }
 
 /******************************************************************************/
@@ -227,11 +369,29 @@ usbd_cdc_itf_t *usb_vcp_get(int idx) {
 */
 
 STATIC mp_obj_t pyb_usb_mode(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+    enum {
+        ARG_mode, ARG_port, ARG_vid, ARG_pid,
+        #if MICROPY_HW_USB_MSC
+        ARG_msc,
+        #endif
+        #if MICROPY_HW_USB_HID
+        ARG_hid,
+        #endif
+        #if USBD_SUPPORT_HS_MODE
+        ARG_high_speed
+        #endif
+    };
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_mode, MP_ARG_REQUIRED | MP_ARG_OBJ, {.u_rom_obj = MP_ROM_PTR(&mp_const_none_obj)} },
+        { MP_QSTR_port, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = -1} },
         { MP_QSTR_vid, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = USBD_VID} },
         { MP_QSTR_pid, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = -1} },
+        #if MICROPY_HW_USB_MSC
+        { MP_QSTR_msc, MP_ARG_KW_ONLY | MP_ARG_OBJ, {.u_rom_obj = MP_ROM_PTR(&mp_const_empty_tuple_obj)} },
+        #endif
+        #if MICROPY_HW_USB_HID
         { MP_QSTR_hid, MP_ARG_KW_ONLY | MP_ARG_OBJ, {.u_rom_obj = MP_ROM_PTR(&pyb_usb_hid_mouse_obj)} },
+        #endif
         #if USBD_SUPPORT_HS_MODE
         { MP_QSTR_high_speed, MP_ARG_KW_ONLY | MP_ARG_BOOL, {.u_bool = false} },
         #endif
@@ -243,7 +403,7 @@ STATIC mp_obj_t pyb_usb_mode(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
         return MP_OBJ_NEW_QSTR(MP_QSTR_host);
     #else
         uint8_t mode = USBD_GetMode(&usb_device.usbd_cdc_msc_hid_state);
-        switch (mode) {
+        switch (mode & USBD_MODE_IFACE_MASK) {
             case USBD_MODE_CDC:
                 return MP_OBJ_NEW_QSTR(MP_QSTR_VCP);
             case USBD_MODE_MSC:
@@ -270,14 +430,14 @@ STATIC mp_obj_t pyb_usb_mode(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
     pyb_usb_flags |= PYB_USB_FLAG_USB_MODE_CALLED;
 
     // check if user wants to disable the USB
-    if (args[0].u_obj == mp_const_none) {
+    if (args[ARG_mode].u_obj == mp_const_none) {
         // disable usb
         pyb_usb_dev_deinit();
         return mp_const_none;
     }
 
     // get mode string
-    const char *mode_str = mp_obj_str_get_str(args[0].u_obj);
+    const char *mode_str = mp_obj_str_get_str(args[ARG_mode].u_obj);
 
 #if defined(USE_HOST_MODE)
 
@@ -296,38 +456,65 @@ STATIC mp_obj_t pyb_usb_mode(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
     // get the VID, PID and USB mode
     // note: PID=-1 means select PID based on mode
     // note: we support CDC as a synonym for VCP for backward compatibility
-    uint16_t vid = args[1].u_int;
-    uint16_t pid = args[2].u_int;
-    usb_device_mode_t mode;
+    uint16_t vid = args[ARG_vid].u_int;
+    mp_int_t pid = args[ARG_pid].u_int;
+    uint8_t mode;
     if (strcmp(mode_str, "CDC+MSC") == 0 || strcmp(mode_str, "VCP+MSC") == 0) {
-        if (args[2].u_int == -1) {
+        if (pid == -1) {
             pid = USBD_PID_CDC_MSC;
         }
         mode = USBD_MODE_CDC_MSC;
-    #if MICROPY_HW_USB_ENABLE_CDC2
+    } else if (strcmp(mode_str, "VCP+MSC+HID") == 0) {
+        if (pid == -1) {
+            pid = USBD_PID_CDC_MSC_HID;
+        }
+        mode = USBD_MODE_CDC_MSC_HID;
+    #if MICROPY_HW_USB_CDC_NUM >= 2
     } else if (strcmp(mode_str, "VCP+VCP") == 0) {
-        if (args[2].u_int == -1) {
+        if (pid == -1) {
             pid = USBD_PID_CDC2;
         }
         mode = USBD_MODE_CDC2;
     } else if (strcmp(mode_str, "VCP+VCP+MSC") == 0) {
-        if (args[2].u_int == -1) {
+        if (pid == -1) {
             pid = USBD_PID_CDC2_MSC;
         }
         mode = USBD_MODE_CDC2_MSC;
+    } else if (strcmp(mode_str, "2xVCP+MSC+HID") == 0) {
+        if (pid == -1) {
+            pid = USBD_PID_CDC2_MSC_HID;
+        }
+        mode = USBD_MODE_CDC2_MSC_HID;
+    #endif
+    #if MICROPY_HW_USB_CDC_NUM >= 3
+    } else if (strcmp(mode_str, "3xVCP") == 0) {
+        if (pid == -1) {
+            pid = USBD_PID_CDC3;
+        }
+        mode = USBD_MODE_CDC3;
+    } else if (strcmp(mode_str, "3xVCP+MSC") == 0) {
+        if (pid == -1) {
+            pid = USBD_PID_CDC3_MSC;
+        }
+        mode = USBD_MODE_CDC3_MSC;
+    } else if (strcmp(mode_str, "3xVCP+MSC+HID") == 0) {
+        if (pid == -1) {
+            pid = USBD_PID_CDC3_MSC_HID;
+        }
+        mode = USBD_MODE_CDC3_MSC_HID;
     #endif
     } else if (strcmp(mode_str, "CDC+HID") == 0 || strcmp(mode_str, "VCP+HID") == 0) {
-        if (args[2].u_int == -1) {
+        if (pid == -1) {
             pid = USBD_PID_CDC_HID;
         }
         mode = USBD_MODE_CDC_HID;
     } else if (strcmp(mode_str, "CDC") == 0 || strcmp(mode_str, "VCP") == 0) {
-        if (args[2].u_int == -1) {
+        if (pid == -1) {
             pid = USBD_PID_CDC;
         }
         mode = USBD_MODE_CDC;
     } else if (strcmp(mode_str, "MSC") == 0) {
-        if (args[2].u_int == -1) {
+        if (pid == -1) {
             pid = USBD_PID_MSC;
         }
         mode = USBD_MODE_MSC;
@@ -335,11 +522,40 @@ STATIC mp_obj_t pyb_usb_mode(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
         goto bad_mode;
     }
 
+    // Get MSC logical units
+    size_t msc_n = 0;
+    const void *msc_unit[USBD_MSC_MAX_LUN];
+    #if MICROPY_HW_USB_MSC
+    if (mode & USBD_MODE_IFACE_MSC) {
+        mp_obj_t *items;
+        mp_obj_get_array(args[ARG_msc].u_obj, &msc_n, &items);
+        if (msc_n > USBD_MSC_MAX_LUN) {
+            mp_raise_ValueError("too many logical units");
+        }
+        for (size_t i = 0; i < msc_n; ++i) {
+            mp_obj_type_t *type = mp_obj_get_type(items[i]);
+            if (type == &pyb_flash_type
+                #if MICROPY_HW_ENABLE_SDCARD
+                || type == &pyb_sdcard_type
+                #endif
+                #if MICROPY_HW_ENABLE_MMCARD
+                || type == &pyb_mmcard_type
+                #endif
+                ) {
+                msc_unit[i] = type;
+            } else {
+                mp_raise_ValueError("unsupported logical unit");
+            }
+        }
+    }
+    #endif
+
     // get hid info if user selected such a mode
     USBD_HID_ModeInfoTypeDef hid_info;
-    if (mode & USBD_MODE_HID) {
+    #if MICROPY_HW_USB_HID
+    if (mode & USBD_MODE_IFACE_HID) {
         mp_obj_t *items;
-        mp_obj_get_array_fixed_n(args[3].u_obj, 5, &items);
+        mp_obj_get_array_fixed_n(args[ARG_hid].u_obj, 5, &items);
         hid_info.subclass = mp_obj_get_int(items[0]);
         hid_info.protocol = mp_obj_get_int(items[1]);
         hid_info.max_packet_len = mp_obj_get_int(items[2]);
@@ -352,15 +568,22 @@ STATIC mp_obj_t pyb_usb_mode(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
         // need to keep a copy of this so report_desc does not get GC'd
         MP_STATE_PORT(pyb_hid_report_desc) = items[4];
     }
+    #endif
 
     #if USBD_SUPPORT_HS_MODE
-    if (args[4].u_bool) {
+    if (args[ARG_high_speed].u_bool) {
         mode |= USBD_MODE_HIGH_SPEED;
     }
     #endif
 
+    // Work out which port/peripheral to use, either user supplied or auto detect
+    int dev_id = args[ARG_port].u_int;
+    if (dev_id == -1) {
+        dev_id = pyb_usb_dev_detect();
+    }
+
     // init the USB device
-    if (!pyb_usb_dev_init(vid, pid, mode, &hid_info)) {
+    if (!pyb_usb_dev_init(dev_id, vid, pid, mode, msc_n, msc_unit, &hid_info)) {
         goto bad_mode;
     }
 
@@ -388,18 +611,35 @@ typedef struct _pyb_usb_vcp_obj_t {
     usbd_cdc_itf_t *cdc_itf;
 } pyb_usb_vcp_obj_t;
 
-const pyb_usb_vcp_obj_t pyb_usb_vcp_obj = {{&pyb_usb_vcp_type}, &usb_device.usbd_cdc_itf};
-#if MICROPY_HW_USB_ENABLE_CDC2
-STATIC const pyb_usb_vcp_obj_t pyb_usb_vcp2_obj = {{&pyb_usb_vcp_type}, &usb_device.usbd_cdc2_itf};
-#endif
+const pyb_usb_vcp_obj_t pyb_usb_vcp_obj[MICROPY_HW_USB_CDC_NUM] = {
+    {{&pyb_usb_vcp_type}, &usb_device.usbd_cdc_itf[0]},
+    #if MICROPY_HW_USB_CDC_NUM >= 2
+    {{&pyb_usb_vcp_type}, &usb_device.usbd_cdc_itf[1]},
+    #endif
+    #if MICROPY_HW_USB_CDC_NUM >= 3
+    {{&pyb_usb_vcp_type}, &usb_device.usbd_cdc_itf[2]},
+    #endif
+};
+
+STATIC void pyb_usb_vcp_init0(void) {
+    // Activate USB_VCP(0) on dupterm slot 1 for the REPL
+    MP_STATE_VM(dupterm_objs[1]) = MP_OBJ_FROM_PTR(&pyb_usb_vcp_obj[0]);
+    usb_vcp_attach_to_repl(&pyb_usb_vcp_obj[0], true);
+}
 
 STATIC void pyb_usb_vcp_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
-    int id = ((pyb_usb_vcp_obj_t*)MP_OBJ_TO_PTR(self_in))->cdc_itf - &usb_device.usbd_cdc_itf;
+    int id = ((pyb_usb_vcp_obj_t*)MP_OBJ_TO_PTR(self_in))->cdc_itf - &usb_device.usbd_cdc_itf[0];
     mp_printf(print, "USB_VCP(%u)", id);
 }
 
 void usb_vcp_attach_to_repl(const pyb_usb_vcp_obj_t *self, bool attached) {
     self->cdc_itf->attached_to_repl = attached;
+    if (attached) {
+        // Default behavior is non-blocking when attached to repl
+        self->cdc_itf->flow &= ~USBD_CDC_FLOWCONTROL_CTS;
+    } else {
+        self->cdc_itf->flow |= USBD_CDC_FLOWCONTROL_CTS;
+    }
 }
 
 /// \classmethod \constructor()
@@ -411,12 +651,8 @@ STATIC mp_obj_t pyb_usb_vcp_make_new(const mp_obj_type_t *type, size_t n_args, s
     // TODO raise exception if USB is not configured for VCP
 
     int id = (n_args == 0) ? 0 : mp_obj_get_int(args[0]);
-    if (id == 0) {
-        return MP_OBJ_FROM_PTR(&pyb_usb_vcp_obj);
-    #if MICROPY_HW_USB_ENABLE_CDC2
-    } else if (id == 1) {
-        return MP_OBJ_FROM_PTR(&pyb_usb_vcp2_obj);
-    #endif
+    if (0 <= id && id < MICROPY_HW_USB_CDC_NUM) {
+        return MP_OBJ_FROM_PTR(&pyb_usb_vcp_obj[id]);
     } else {
         mp_raise_ValueError(NULL);
     }
@@ -457,7 +693,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_usb_vcp_isconnected_obj, pyb_usb_vcp_isconn
 
 // deprecated in favour of USB_VCP.isconnected
 STATIC mp_obj_t pyb_have_cdc(void) {
-    return pyb_usb_vcp_isconnected(MP_OBJ_FROM_PTR(&pyb_usb_vcp_obj));
+    return pyb_usb_vcp_isconnected(MP_OBJ_FROM_PTR(&pyb_usb_vcp_obj[0]));
 }
 MP_DEFINE_CONST_FUN_OBJ_0(pyb_have_cdc_obj, pyb_have_cdc);
 
@@ -561,6 +797,7 @@ STATIC const mp_rom_map_elem_t pyb_usb_vcp_locals_dict_table[] = {
 
     // class constants
     { MP_ROM_QSTR(MP_QSTR_RTS), MP_ROM_INT(USBD_CDC_FLOWCONTROL_RTS) },
+    { MP_ROM_QSTR(MP_QSTR_CTS), MP_ROM_INT(USBD_CDC_FLOWCONTROL_CTS) },
 };
 
 STATIC MP_DEFINE_CONST_DICT(pyb_usb_vcp_locals_dict, pyb_usb_vcp_locals_dict_table);
@@ -578,18 +815,13 @@ STATIC mp_uint_t pyb_usb_vcp_read(mp_obj_t self_in, void *buf, mp_uint_t size, i
 
 STATIC mp_uint_t pyb_usb_vcp_write(mp_obj_t self_in, const void *buf, mp_uint_t size, int *errcode) {
     pyb_usb_vcp_obj_t *self = MP_OBJ_TO_PTR(self_in);
-    if (self->cdc_itf->attached_to_repl) {
-        usbd_cdc_tx_always(self->cdc_itf, (const byte*)buf, size);
-        return size;
-    } else {
-        int ret = usbd_cdc_tx(self->cdc_itf, (const byte*)buf, size, 0);
-        if (ret == 0) {
-            // return EAGAIN error to indicate non-blocking
-            *errcode = MP_EAGAIN;
-            return MP_STREAM_ERROR;
-        }
-        return ret;
+    int ret = usbd_cdc_tx_flow(self->cdc_itf, (const byte*)buf, size);
+    if (ret == 0) {
+        // return EAGAIN error to indicate non-blocking
+        *errcode = MP_EAGAIN;
+        return MP_STREAM_ERROR;
     }
+    return ret;
 }
 
 STATIC mp_uint_t pyb_usb_vcp_ioctl(mp_obj_t self_in, mp_uint_t request, uintptr_t arg, int *errcode) {
@@ -631,6 +863,8 @@ const mp_obj_type_t pyb_usb_vcp_type = {
 /******************************************************************************/
 // MicroPython bindings for USB HID
 
+#if MICROPY_HW_USB_HID
+
 typedef struct _pyb_usb_hid_obj_t {
     mp_obj_base_t base;
     usb_device_t *usb_dev;
@@ -676,6 +910,11 @@ STATIC mp_obj_t pyb_usb_hid_recv(size_t n_args, const mp_obj_t *args, mp_map_t *
     // receive the data
     int ret = usbd_hid_rx(&self->usb_dev->usbd_hid_itf, vstr.len, (uint8_t*)vstr.buf, vals[1].u_int);
 
+    if (ret < 0) {
+        // error, just return 0/empty bytes
+        ret = 0;
+    }
+
     // return the received data
     if (o_ret != MP_OBJ_NULL) {
         return mp_obj_new_int(ret); // number of bytes read into given buffer
@@ -759,6 +998,8 @@ const mp_obj_type_t pyb_usb_hid_type = {
     .locals_dict = (mp_obj_dict_t*)&pyb_usb_hid_locals_dict,
 };
 
+#endif // MICROPY_HW_USB_HID
+
 /******************************************************************************/
 // code for experimental USB OTG support
 
