@@ -38,6 +38,11 @@
 #define USBD_PID_MSC     (0x9803)
 #define USBD_PID_CDC2_MSC (0x9804)
 #define USBD_PID_CDC2    (0x9805)
+#define USBD_PID_CDC3    (0x9806)
+#define USBD_PID_CDC3_MSC (0x9807)
+#define USBD_PID_CDC_MSC_HID (0x9808)
+#define USBD_PID_CDC2_MSC_HID (0x9809)
+#define USBD_PID_CDC3_MSC_HID (0x980a)
 
 typedef enum {
     PYB_USB_STORAGE_MEDIUM_NONE = 0,
@@ -58,14 +63,14 @@ extern const struct _mp_rom_obj_tuple_t pyb_usb_hid_mouse_obj;
 extern const struct _mp_rom_obj_tuple_t pyb_usb_hid_keyboard_obj;
 extern const mp_obj_type_t pyb_usb_vcp_type;
 extern const mp_obj_type_t pyb_usb_hid_type;
-extern const pyb_usb_vcp_obj_t pyb_usb_vcp_obj;
 
 MP_DECLARE_CONST_FUN_OBJ_KW(pyb_usb_mode_obj);
 MP_DECLARE_CONST_FUN_OBJ_0(pyb_have_cdc_obj); // deprecated
 MP_DECLARE_CONST_FUN_OBJ_1(pyb_hid_send_report_obj); // deprecated
 
 void pyb_usb_init0(void);
-bool pyb_usb_dev_init(uint16_t vid, uint16_t pid, usb_device_mode_t mode, USBD_HID_ModeInfoTypeDef *hid_info);
+int pyb_usb_dev_detect(void);
+bool pyb_usb_dev_init(int dev_id, uint16_t vid, uint16_t pid, uint8_t mode, size_t msc_n, const void *msc_unit, USBD_HID_ModeInfoTypeDef *hid_info);
 void pyb_usb_dev_deinit(void);
 bool usb_vcp_is_enabled(void);
 int usb_vcp_recv_byte(uint8_t *c); // if a byte is available, return 1 and put the byte in *c, else return 0
