@@ -39,7 +39,7 @@
 
 #include "py/mpconfig.h"
 
-#define USBD_MAX_NUM_INTERFACES               4
+#define USBD_MAX_NUM_INTERFACES               8
 #define USBD_MAX_NUM_CONFIGURATION            1
 #define USBD_MAX_STR_DESC_SIZ                 0x100
 #if MICROPY_HW_USB_SELF_POWERED
@@ -49,6 +49,16 @@
 #endif
 #define USBD_DEBUG_LEVEL                      0
 
+// For MCUs with a device-only USB peripheral
+#define USBD_PMA_RESERVE                      (64)
+#define USBD_PMA_NUM_FIFO                     (8)
+
+// For MCUs with multiple OTG USB peripherals
+#define USBD_FS_NUM_TX_FIFO                   (6)
+#define USBD_FS_NUM_FIFO                      (1 + USBD_FS_NUM_TX_FIFO)
+#define USBD_HS_NUM_TX_FIFO                   (9)
+#define USBD_HS_NUM_FIFO                      (1 + USBD_HS_NUM_TX_FIFO)
+
 #endif // MICROPY_INCLUDED_STM32_USBD_CONF_H
 
 /************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
