@@ -119,10 +119,8 @@ extern const struct _mp_obj_module_t mp_module_zsensor;
 
 #if MICROPY_PY_USOCKET
 #define MICROPY_PY_USOCKET_DEF { MP_ROM_QSTR(MP_QSTR_usocket), MP_ROM_PTR(&mp_module_usocket) },
-#define MICROPY_PY_USOCKET_WEAK_DEF { MP_ROM_QSTR(MP_QSTR_socket), MP_ROM_PTR(&mp_module_usocket) },
 #else
 #define MICROPY_PY_USOCKET_DEF
-#define MICROPY_PY_USOCKET_WEAK_DEF
 #endif
 
 #if MICROPY_PY_UTIME
@@ -150,10 +148,6 @@ extern const struct _mp_obj_module_t mp_module_zsensor;
     MICROPY_PY_ZEPHYR_DEF \
     MICROPY_PY_ZSENSOR_DEF \
 
-#define MICROPY_PORT_BUILTIN_MODULE_WEAK_LINKS \
-    { MP_ROM_QSTR(MP_QSTR_time), MP_ROM_PTR(&mp_module_time) }, \
-    MICROPY_PY_USOCKET_WEAK_DEF \
-
 // extra built in names to add to the global namespace
 #define MICROPY_PORT_BUILTINS \
 
