@@ -13,10 +13,6 @@ try:
     ~[]
 except TypeError:
     print('TypeError')
-try:
-    ~bytearray()
-except TypeError:
-    print('TypeError')
 
 # unsupported binary operators
 try:
@@ -31,16 +27,6 @@ try:
     1 in 1
 except TypeError:
     print('TypeError')
-try:
-    bytearray() // 2
-except TypeError:
-    print('TypeError')
-
-# object with buffer protocol needed on rhs
-try:
-    bytearray(1) + 1
-except TypeError:
-    print('TypeError')
 
 # unsupported subscription
 try:
