@@ -21,3 +21,9 @@ print("0000".find('-1', 3))
 print("0000".find('1', 3))
 print("0000".find('1', 4))
 print("0000".find('1', 5))
+print("aaaaaaaaaaa".find("bbb", 9, 2))
+
+try:
+    'abc'.find(1)
+except TypeError:
+    print('TypeError')
