@@ -4,8 +4,7 @@ try:
     from math import isfinite, isnan, isinf
 except ImportError:
     print("SKIP")
-    import sys
-    sys.exit()
+    raise SystemExit
 
 test_values = [1, 0, -1, 1.0, 0.0, -1.0, float('NaN'), float('Inf'), 
                -float('NaN'), -float('Inf')]
