@@ -1,10 +1,17 @@
 # test that iterating doesn't use the heap
 try:
     frozenset
-    import array
-except (NameError, ImportError):
+except NameError:
     print("SKIP")
     raise SystemExit
+try:
+    import uarray as array
+except ImportError:
+    try:
+        import array
+    except ImportError:
+        print("SKIP")
+        raise SystemExit
 
 try:
     from micropython import heap_lock, heap_unlock
