@@ -25,7 +25,15 @@ def f4(x1:int, x2:int, x3:int, x4:int):
     print(x1, x2, x3, x4)
 f4(1, 2, 3, 4)
 
-# only up to 4 arguments currently supported
+@micropython.viper
+def f5(x1:int, x2:int, x3:int, x4:int, x5:int):
+    print(x1, x2, x3, x4, x5)
+f5(1, 2, 3, 4, 5)
+
+@micropython.viper
+def f6(x1:int, x2:int, x3:int, x4:int, x5:int, x6:int):
+    print(x1, x2, x3, x4, x5, x6)
+f6(1, 2, 3, 4, 5, 6)
 
 # test compiling *x, **x, * args (currently unsupported at runtime)
 @micropython.viper
