@@ -1,5 +1,9 @@
 import pyb
 
+if not hasattr(pyb, 'Accel'):
+    print('SKIP')
+    raise SystemExit
+
 accel = pyb.Accel()
 print(accel)
 accel.x()
